/**

Name: 		toggle-switch-led-1.0

Author: 	Ross Witherby VK3FPAA - http://someguynamedross.com

Date:		15.12.2012

This *very* basic PIC project toggles 2 LED's and an output which is fed to another PIC (A 16F88,the controller used in a JUMA RX1 radio)

A very basic (yet effective) debounce scheme is used here, internal pull-ups are enabled so all that is needed is a current limiting resistor (I used 1k) and a momentory switch to ground.

The LED's will need a current limiting resistor, these will need to be chosen based on your LED's

This code is based very heavily on the tutorials at http://www.gooligum.com.au/tutorials.html

**/

#include 	<xc.h>
#include 	<stdint.h>

#define _XTAL_FREQ 4000000 // Internal 4MHz R/C oscillator is used

__CONFIG(MCLRE_OFF & CP_OFF & CPD_OFF & BOREN_OFF & WDTE_OFF & PWRTE_OFF & FOSC_INTRCIO);

/*** Pin Assignments ***/

#define SW_in GPIObits.GP5		// pin 2
#define sVFOA_led sGPIO.GP1		// pin 6
#define sVFOB_led sGPIO.GP2		// pin 5
#define sMCU_out sGPIO.GP4		// pin 3

/*** Global Variables ***/

volatile union {
	uint8_t port;
	struct {
		unsigned GP0 : 1;
		unsigned GP1 : 1;
		unsigned GP2 : 1;
		unsigned GP3 : 1;
		unsigned GP4 : 1;
		unsigned GP5 : 1;
	};
} sGPIO;

/*** Main Program ***/

void main() {
	uint8_t db_count;

	OPTION_REGbits.nGPPU = 0;	// enable pull-ups
	WPU = 0b100000;				// set pull-up on GP5
	TRISIO = 0b100000;			// GP5 as input, GP0,1,2,3 & 4 as outputs
	sVFOA_led = 1;				// set GP1 to high
	sVFOB_led = 0;				// set GP2 to low
	sMCU_out = 1;				// set GP4 to high
	GPIO = sGPIO.port;			// update GPIO from shadow 
	
	for (;;) {
		for (db_count = 0; db_count <= 10; db_count++) {	// debounce code, waits for GP5 to go low
			__delay_ms(1);
			if(SW_in == 1)
				db_count = 0;
		}
		
		sVFOA_led = ~sVFOA_led;	// toggle sVFOA_led output
		sVFOB_led = ~sVFOB_led;	// toggle sVFOB_led output
		sMCU_out = ~sMCU_out;	// toggle sMCU_out output
		GPIO = sGPIO.port;		// update GPIO from shadow

		for (db_count = 0; db_count <= 10; db_count++) {	// debounce code, waits for GP5 to go high
			__delay_ms(1);
			if(SW_in == 0)
				db_count = 0;
		}
	}
}